// Delphi 3.0 Implementation of IShellView Interface Object
//
// This object must be implemented in a separate object, since the Shell
// can create multiple views of the same folder. It expects that all of these
// views can act independently of each other and present different
// representations of the same folder.
//
// (C)1997 Naleco Research Inc.
// Written by Dr. John F. Lam

(*******************************************************************************
**
**  Updated to reflect new ViewForm Constructor of TView form - ajs 11Jan99
**  Stripped out superflous code as a result of inheriting from
**    TInterfacedObject instead of TComObject (see ShellFolder.pas).
**
**   - ajs 14Jan99
**
*******************************************************************************)
unit ShellView;

interface

uses Windows, ActiveX, CommCtrl, ShellAPI, ShlObj, ViewForm;

type
  TShellViewImpl = class( TInterfacedObject, IShellView )
  private
    FFolderSettings: TFolderSettings;
    FShellBrowser: IShellBrowser;
    FHWndParent: HWND;
    FForm: TView;

  public
    constructor Create;

    // IOleWindow Methods
    function GetWindow(out wnd: HWnd): HResult; stdcall;
    function ContextSensitiveHelp(fEnterMode: BOOL): HResult; stdcall;

    // IShellView Methods
    function TranslateAccelerator(var Msg: TMsg): HResult; stdcall;
    function EnableModeless(Enable: Boolean): HResult; stdcall;
    function UIActivate(State: UINT): HResult; stdcall;
    function Refresh: HResult; stdcall;
    function CreateViewWindow(PrevView: IShellView;
      var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
      var Rect: TRect; out Wnd: HWND): HResult; stdcall;
    function DestroyViewWindow: HResult; stdcall;
    function GetCurrentInfo(out FolderSettings: TFolderSettings): HResult; stdcall;
    function AddPropertySheetPages(Reseved: DWORD;
      lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult; stdcall;
    function SaveViewState: HResult; stdcall;
    function SelectItem(pidl: PItemIDList; flags: UINT): HResult; stdcall;
    function GetItemObject(Item: UINT; iid: TIID; IPtr: Pointer): HResult; stdcall;

    property ShellBrowser: IShellBrowser read FShellBrowser;

  end;

implementation

uses
  RegisterExtension, Classes;

(*
**  We can create any objects we need here.
**  Initialise pointers to nil.
*)
constructor TShellViewImpl.Create;
begin
  inherited Create;
  FForm := nil;
  FShellBrowser := nil;
end;

////////////////////////////////////////////////////////////////////////////////
// IOleWindow Implementation

function TShellViewImpl.GetWindow(out wnd: HWnd): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IOleWindow.GetWindow' ) );
  Wnd := FForm.Handle;
  Result := NOERROR;
end;

function TShellViewImpl.ContextSensitiveHelp(fEnterMode: BOOL): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IOleWindow.ContextSensitiveHelp' ) );
  Result := E_NOTIMPL;
end;

////////////////////////////////////////////////////////////////////////////////
// IShellView Implementation

// This function is called by the Shell to give us first crack at translating
// an accelerator key. We don't currently support accelerators, so we just
// return NOERROR. The documentation regarding this is INCORRECT! We must
// return E_NOTIMPL and NOT NOERROR ... otherwise Explorer will hang.

function TShellViewImpl.TranslateAccelerator(var Msg: TMsg): HResult; stdcall;
begin
  Result := E_NOTIMPL;
end;

function TShellViewImpl.EnableModeless(Enable: Boolean): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.EnableModeless' ) );
  Result := E_NOTIMPL;
end;

function TShellViewImpl.UIActivate(State: UINT): HResult; stdcall;
var
  S: String;
begin
  case TSVUIAEnums(State) of
  SVUIA_DEACTIVATE:
    S := 'Deactivate view';
  SVUIA_ACTIVATE_NOFOCUS:
    S := 'Activate view without focus';
  SVUIA_ACTIVATE_FOCUS:
      S := 'Activate view with focus';
  SVUIA_INPLACEACTIVATE:
    S := 'Activate view for inplace-activation within ActiveX control';
  end;
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.UIActivate: ' + S ) );
  Result := NOERROR;
end;

(*
**  This is called whenever Explorer needs to update the right-hand pane (when
**  the user presses F5 for example)
*)
function TShellViewImpl.Refresh: HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.Refresh' ) );
  Result := E_NOTIMPL;
end;

// This function is the third function in the sequence called by Explorer.
// Once our TShellViewImpl object has been created, this function is called
// to create the actual window that contains our view.
//
// Two possible modes of action are possible:
//
// (1) Create a new view window that contains our view when outside of
//     Explorer (just like a folder window being opened via the desktop).
//
// (2) Create a new right hand pane window in Explorer

function TShellViewImpl.CreateViewWindow(PrevView: IShellView;
  var FolderSettings: TFolderSettings; ShellBrowser: IShellBrowser;
  var Rect: TRect; out Wnd: HWND): HResult; stdcall;
begin
  // Save the folder settings passed
  FFolderSettings := FolderSettings;
  FShellBrowser := ShellBrowser;

  // Get the window handle of Explorer's Parent Window
  FShellBrowser.GetWindow( FHWndParent );

  // Create our Form -  we need to pass these references for the form to use
  //  to notify the shell when our form gets focus - ajs 11Jan99
  try
    FForm := TView.CreateShView(nil, FShellBrowser, Self as IShellView);
    Wnd := FForm.Handle;
    SetParent( Wnd, FHWndParent );

    with FForm do
    begin
      SetWindowPos( Handle, HWND_TOP, Rect.Left, Rect.Top,
        Rect.Right - Rect.Left, Rect.Bottom - Rect.Top, SWP_SHOWWINDOW );
      Show;
    end;

    if Wnd <> 0 then
      Result := NOERROR
    else
      Result := E_UNEXPECTED;

  except
    Result := E_UNEXPECTED;
  end;
end;

function TShellViewImpl.DestroyViewWindow: HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.DestroyViewWindow' ) );
  FForm.Free;
  FForm := nil;
  Result := NOERROR;
end;

function TShellViewImpl.GetCurrentInfo(out FolderSettings: TFolderSettings): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.GetCurrentInfo' ) );
  Result := E_NOTIMPL;
end;

function TShellViewImpl.AddPropertySheetPages(Reseved: DWORD;
  lpfnAddPage: TFNAddPropSheetPage; lParam: LPARAM): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.AddPropertySheetPages' ) );
  Result := E_NOTIMPL;
end;

function TShellViewImpl.SaveViewState: HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.SaveViewState' ) );
  Result := E_NOTIMPL;
end;

function TShellViewImpl.SelectItem(pidl: PItemIDList; flags: UINT): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.SelectItem' ) );
  Result := E_NOTIMPL;
end;

function TShellViewImpl.GetItemObject(Item: UINT; iid: TIID; IPtr: Pointer): HResult; stdcall;
begin
  FShellBrowser.SetStatusTextSB( StringToOleStr( 'IShellView.GetItemObject' ) );
  Result := E_NOTIMPL;
end;

end.
